﻿using System;
using SDX.Compiler;
using Mono.Cecil;
using Mono.Cecil.Cil;
using System.Linq;
using System.Collections.Generic;

public class CellRuleHubPositionSet : IPatcherMod
{
    public bool Link(ModuleDefinition gameModule, ModuleDefinition modModule)
    {
        InsertHubCellCstr(gameModule, modModule);
        InsertAddHubRuleGridPos(gameModule, modModule);
        InsertSetHubRuleAttribute(gameModule, modModule);
        InsertHubCellGenerateDetail(gameModule, modModule);
        InsertHGSocketWW(gameModule, modModule);
        return true;
    }
    public void InsertHGSocketWW(ModuleDefinition gameModule, ModuleDefinition modModule)
    {
        TypeDefinition classDef = gameModule.Types.First(d => d.Name == "HGSockets" && d.Namespace == "RWG2.HubGenerators");
        MethodDefinition methodDef = classDef.Methods.First(d => d.Name == "WW");
        TypeDefinition hgAbstruct = gameModule.Types.First(d => d.Name == "HGAbstract" && d.Namespace == "RWG2.HubGenerators");
        FieldDefinition hubCellField = hgAbstruct.Fields.First(d => d.Name == "hubCell");
        Instruction insertPos1 = null;
        Instruction jumpToPos1 = null;
        Instruction insertPos2 = null;
        Instruction jumpToPos2 = null;
        foreach (Instruction inst in methodDef.Body.Instructions)
        {
            if (inst.OpCode == OpCodes.Ldc_I4_S && inst.Operand.GetType() == typeof(sbyte) && ((sbyte)inst.Operand) == 10)
            {
                insertPos1 = inst;
            }
            if (inst.OpCode == OpCodes.Callvirt && inst.Operand.GetType() == typeof(MethodReference) && ((MethodReference)inst.Operand).FullName == "System.Void System.Collections.Generic.List`1<UnityEngine.Vector2>::Insert(System.Int32,!0)")
            {
                jumpToPos1 = inst.Next;
            }
            if (inst.OpCode == OpCodes.Call && inst.Operand.GetType() == typeof(MethodDefinition) && ((MethodDefinition)inst.Operand).FullName == "System.Int32 RWG2.HubGenerators.HGSockets::floodFill(UnityEngine.Vector2,System.Collections.Generic.List`1<Vector2i>&)")
            {
                insertPos2 = inst.Previous.Previous.Previous.Previous.Previous.Previous.Previous.Previous.Previous;
                jumpToPos2 = inst.Next.Next.Next.Next.Next.Next;
            }
        }
        ILProcessor proc1 = methodDef.Body.GetILProcessor();

        TypeDefinition cellRuleHubPositionSetMethod = modModule.Types.First(d => d.Name == "CellRuleHubPositionSetMethod");
        MethodDefinition isSetHubOffset = cellRuleHubPositionSetMethod.Methods.First(d => d.Name == "IsSetHubOffset");
        MethodReference isSetHubOffsetRef = gameModule.Import(isSetHubOffset);

        MethodDefinition getHubPositionList = cellRuleHubPositionSetMethod.Methods.First(d => d.Name == "GetHubPositionList");
        MethodReference getHubPositionListRef = gameModule.Import(getHubPositionList);
        

        proc1.InsertBefore(insertPos1, Instruction.Create(OpCodes.Ldarg_0));
        proc1.InsertBefore(insertPos1, Instruction.Create(OpCodes.Ldfld, hubCellField));
        proc1.InsertBefore(insertPos1, Instruction.Create(OpCodes.Call, isSetHubOffsetRef));
        proc1.InsertBefore(insertPos1, Instruction.Create(OpCodes.Brfalse_S, insertPos1));
        proc1.InsertBefore(insertPos1, Instruction.Create(OpCodes.Ldarg_0));
        proc1.InsertBefore(insertPos1, Instruction.Create(OpCodes.Ldfld, hubCellField));
        proc1.InsertBefore(insertPos1, Instruction.Create(OpCodes.Call, getHubPositionListRef));
        proc1.InsertBefore(insertPos1, Instruction.Create(OpCodes.Stloc_3));

        proc1.InsertBefore(insertPos1, Instruction.Create(OpCodes.Br, jumpToPos1));

        MethodDefinition floodFillSet = cellRuleHubPositionSetMethod.Methods.First(d => d.Name == "floodFillSet");
        MethodReference floodFillSetRef = gameModule.Import(floodFillSet);

        TypeDefinition hubCell = gameModule.Types.First(d => d.Name == "HubCell" && d.Namespace == "RWG2");
        TypeDefinition hubPathNode = hubCell.NestedTypes.First(d => d.Name == "HubPathNode");
        FieldDefinition areaField = hubPathNode.Fields.First(d => d.Name == "Area");
        FieldDefinition hubFoundField = hubCell.Fields.First(d => d.Name == "hubFound");
        FieldDefinition hubNodesField = hubCell.Fields.First(d => d.Name == "hubNodes");

        TypeReference rect = gameModule.Import(typeof(UnityEngine.Rect));
        TypeDefinition typeDef = rect.Resolve();
        MethodDefinition getCenter = typeDef.Methods.First(d => d.Name == "get_center");
        MethodReference getCenterRef = gameModule.Import(getCenter);

        proc1.InsertBefore(insertPos2, Instruction.Create(OpCodes.Ldarg_0));
        proc1.InsertBefore(insertPos2, Instruction.Create(OpCodes.Ldfld, hubCellField));
        proc1.InsertBefore(insertPos2, Instruction.Create(OpCodes.Call, isSetHubOffsetRef));
        proc1.InsertBefore(insertPos2, Instruction.Create(OpCodes.Brfalse_S, insertPos2));

        proc1.InsertBefore(insertPos2, Instruction.Create(OpCodes.Ldarg_0));
        proc1.InsertBefore(insertPos2, Instruction.Create(OpCodes.Ldloca_S, proc1.Body.Variables[7]));
        proc1.InsertBefore(insertPos2, Instruction.Create(OpCodes.Ldflda, areaField));
        proc1.InsertBefore(insertPos2, Instruction.Create(OpCodes.Call, getCenterRef));
        proc1.InsertBefore(insertPos2, Instruction.Create(OpCodes.Ldarg_0));
        proc1.InsertBefore(insertPos2, Instruction.Create(OpCodes.Ldfld, hubCellField));
        proc1.InsertBefore(insertPos2, Instruction.Create(OpCodes.Ldflda, hubNodesField));
        
        proc1.InsertBefore(insertPos2, Instruction.Create(OpCodes.Call, floodFillSetRef));
        proc1.InsertBefore(insertPos2, Instruction.Create(OpCodes.Pop));
        proc1.InsertBefore(insertPos2, Instruction.Create(OpCodes.Ldarg_0));
        proc1.InsertBefore(insertPos2, Instruction.Create(OpCodes.Ldfld, hubCellField));
        proc1.InsertBefore(insertPos2, Instruction.Create(OpCodes.Ldc_I4_1));
        proc1.InsertBefore(insertPos2, Instruction.Create(OpCodes.Stfld, hubFoundField));

        proc1.InsertBefore(insertPos2, Instruction.Create(OpCodes.Br, jumpToPos2));

    }
    public void InsertHubCellGenerateDetail(ModuleDefinition gameModule, ModuleDefinition modModule)
    {
        TypeDefinition classDef = gameModule.Types.First(d => d.Name == "HubCell" && d.Namespace == "RWG2");
        MethodDefinition methodDef = classDef.Methods.First(d => d.Name == "GenerateDetails");
        FieldDefinition cellRuleField = classDef.Fields.First(d => d.Name == "CellRule");

        Instruction insertPosition = null;
        Instruction jumpToPosition = null;
        foreach (Instruction inst in methodDef.Body.Instructions)
        {
            if (inst.OpCode == OpCodes.Ldfld && inst.Operand.GetType() == typeof(FieldDefinition) && ((FieldDefinition)inst.Operand).FullName == "RWG2.Rules.HubRule RWG2.HubCell::HubRule")
            {
                if (inst.Next.OpCode == OpCodes.Brfalse)
                {
                    insertPosition = inst.Next.Next.Next.Next;
                }
            }
            if (inst.OpCode == OpCodes.Callvirt && inst.Operand.GetType() == typeof(MethodDefinition) && ((MethodDefinition)inst.Operand).FullName == "System.Boolean RWG2.Rules.HubRule::get_HasHubLayout()")
            {
                jumpToPosition = inst.Previous.Previous;
            }
        }
        TypeDefinition cellRuleHubPositionSetMethod = modModule.Types.First(d => d.Name == "CellRuleHubPositionSetMethod");
        MethodDefinition isSetHubOffset = cellRuleHubPositionSetMethod.Methods.First(d => d.Name == "IsSetHubOffset");
        MethodReference isSetHubOffsetRef = gameModule.Import(isSetHubOffset);

        MethodDefinition getHubPosition = cellRuleHubPositionSetMethod.Methods.First(d => d.Name == "GetHubPosition");
        MethodReference getHubPositionRef = gameModule.Import(getHubPosition);

        ILProcessor proc1 = methodDef.Body.GetILProcessor();

        proc1.InsertBefore(insertPosition, Instruction.Create(OpCodes.Ldarg_0));
        proc1.InsertBefore(insertPosition, Instruction.Create(OpCodes.Call, isSetHubOffsetRef));
        proc1.InsertBefore(insertPosition, Instruction.Create(OpCodes.Brfalse_S, insertPosition));
        proc1.InsertBefore(insertPosition, Instruction.Create(OpCodes.Ldarg_0));
        proc1.InsertBefore(insertPosition, Instruction.Create(OpCodes.Call, getHubPositionRef));
        proc1.InsertBefore(insertPosition, Instruction.Create(OpCodes.Stloc_3));
        proc1.InsertBefore(insertPosition, Instruction.Create(OpCodes.Br, jumpToPosition));




    }

    public void InsertSetHubRuleAttribute(ModuleDefinition gameModule, ModuleDefinition modModule)
    {

        TypeDefinition classDef = gameModule.Types.First(d => d.Name == "RulesFromXml" && d.Namespace == "RWG2.Rules");
        MethodDefinition methodDef = classDef.Methods.First(d => d.Name == "WZ");
        Instruction insertPosition = null;
        Instruction insertInitPosition = null;
        foreach (Instruction inst in methodDef.Body.Instructions)
        {
            if (inst.OpCode == OpCodes.Ldstr && inst.Operand.GetType() == typeof(string) && ((string)inst.Operand) == "hub_type")
            {
                insertPosition = inst.Previous.Previous.Previous.Previous.Previous;
            }
            if (inst.OpCode == OpCodes.Stfld && inst.Operand.GetType() == typeof(FieldDefinition) && ((FieldDefinition)inst.Operand).FullName == "System.Int32 RWG2.Rules.HubRule::PathRadius")
            {
                if (inst.Previous.OpCode == OpCodes.Ldc_I4_7)
                {
                    insertInitPosition = inst.Previous.Previous;
                }
            }
        }
        ILProcessor proc1 = methodDef.Body.GetILProcessor();
        TypeDefinition cellRuleHubPositionSetMethod = modModule.Types.First(d => d.Name == "CellRuleHubPositionSetMethod");
        MethodDefinition setHubRuleAttribute = cellRuleHubPositionSetMethod.Methods.First(d => d.Name == "SetHubRuleAttribute");
        MethodReference setHubRuleAttributeRef = gameModule.Import(setHubRuleAttribute);

        proc1.InsertBefore(insertPosition, Instruction.Create(OpCodes.Ldloc_3));
        proc1.InsertBefore(insertPosition, Instruction.Create(OpCodes.Ldloc_S, proc1.Body.Variables[6]));
        proc1.InsertBefore(insertPosition, Instruction.Create(OpCodes.Call, setHubRuleAttributeRef));

        TypeDefinition hubRule = gameModule.Types.First(d => d.Name == "HubRule" && d.Namespace == "RWG2.Rules");
        FieldDefinition isSetHubOffset = hubRule.Fields.First(d => d.Name == "IsSetHubOffset");

        proc1.InsertBefore(insertInitPosition, Instruction.Create(OpCodes.Ldloc_3));
        proc1.InsertBefore(insertInitPosition, Instruction.Create(OpCodes.Ldc_I4_0));
        proc1.InsertBefore(insertInitPosition, Instruction.Create(OpCodes.Stfld, isSetHubOffset));

    }

    public void InsertAddHubRuleGridPos(ModuleDefinition gameModule, ModuleDefinition modModule)
    {

        TypeDefinition classDef = gameModule.Types.First(d => d.Name == "RulesFromXml" && d.Namespace == "RWG2.Rules");
        MethodDefinition methodDef = classDef.Methods.First(d => d.Name == "ZZ");
        List<Instruction> insertList = new List<Instruction>();
        foreach (Instruction inst in methodDef.Body.Instructions)
        {
            if (inst.OpCode == OpCodes.Ldsfld && inst.Operand.GetType() == typeof(FieldReference) && ((FieldReference)inst.Operand).FullName == "System.String System.String::Empty")
            {
                if (inst.Previous.OpCode == OpCodes.Ldloc_S && inst.Previous.Operand.GetType() == typeof(VariableDefinition) && ((VariableDefinition)inst.Previous.Operand).Name == methodDef.Body.Variables[7].Name)
                {
                    insertList.Add(inst.Next.Next.Next.Next.Next);
                }
            }
        }

        ILProcessor proc1 = methodDef.Body.GetILProcessor();
        TypeDefinition cellRuleHubPositionSetMethod = modModule.Types.First(d => d.Name == "CellRuleHubPositionSetMethod");
        MethodDefinition addHubRuleGridPos = cellRuleHubPositionSetMethod.Methods.First(d => d.Name == "AddHubRuleGridPos");

        MethodReference addHubRuleGridPosRef = gameModule.Import(addHubRuleGridPos);
        foreach (Instruction insertPosition in insertList)
        {
            proc1.InsertBefore(insertPosition, Instruction.Create(OpCodes.Ldloc_3));
            proc1.InsertBefore(insertPosition, Instruction.Create(OpCodes.Ldloc_S, proc1.Body.Variables[7]));
            proc1.InsertBefore(insertPosition, Instruction.Create(OpCodes.Ldloc_S, proc1.Body.Variables[6]));
            proc1.InsertBefore(insertPosition, Instruction.Create(OpCodes.Call, addHubRuleGridPosRef));
        }


    }

    public void InsertHubCellCstr(ModuleDefinition gameModule, ModuleDefinition modModule)
    {
        //            CheckMethodNameSpace(md, "HubCell", ".ctor", 1,"RWG2");
        TypeDefinition classDef = gameModule.Types.First(d => d.Name == "HubCell" && d.Namespace == "RWG2");
        MethodDefinition methodDef = classDef.Methods.First(d => d.Name == ".ctor" && d.Parameters.Count == 1);
        Instruction insertPosition = null;
        Instruction jumpToPosition = null;
        foreach (Instruction inst in methodDef.Body.Instructions)
        {
            if (inst.OpCode == OpCodes.Callvirt && inst.Operand.GetType() == typeof(MethodDefinition) && ((MethodDefinition)inst.Operand).FullName == "RWG2.Rules.HubRule RWG2.Rules.CellRule::GetRandomHubRule(UnityEngine.Vector2)")
            {
                insertPosition = inst.Previous.Previous.Previous.Previous.Previous;
                jumpToPosition = inst.Next.Next;
            }
        }
        TypeDefinition cellRuleHubPositionSetMethod = modModule.Types.First(d => d.Name == "CellRuleHubPositionSetMethod");
        MethodDefinition getGridHubRule = cellRuleHubPositionSetMethod.Methods.First(d => d.Name == "GetGridHubRule");
        MethodReference getGridHubRuleRef = gameModule.Import(getGridHubRule);

        ILProcessor proc1 = methodDef.Body.GetILProcessor();

        proc1.InsertBefore(insertPosition, Instruction.Create(OpCodes.Ldarg_0));
        proc1.InsertBefore(insertPosition, Instruction.Create(OpCodes.Ldarg_1));

        proc1.InsertBefore(insertPosition, Instruction.Create(OpCodes.Call, getGridHubRuleRef));
        proc1.InsertBefore(insertPosition, Instruction.Create(OpCodes.Brtrue_S, jumpToPosition));

    }

    public bool Patch(ModuleDefinition module)
    {
        Console.WriteLine("==CellRuleHubPositionSet Patcher===");
        AddCellRuleFieldDefinition(module);
        SetHGSocketFieldPublic(module);
        return true;
    }
    public void SetHGSocketFieldPublic(ModuleDefinition module)
    {
        TypeDefinition classDef = module.Types.First(d => d.Name == "HGSockets" && d.Namespace == "RWG2.HubGenerators");
        SetFieldToPublic(classDef, "TZ");
        SetFieldToPublic(classDef, "UZ");
        SetFieldToPublic(classDef, "YZ");
        SetFieldToPublic(classDef, "NZ");
        TypeDefinition hgAbstruct = module.Types.First(d => d.Name == "HGAbstract" && d.Namespace == "RWG2.HubGenerators");
        SetFieldToPublic(hgAbstruct, "hubCell");
    }
    public void AddCellRuleFieldDefinition(ModuleDefinition module)
    {
        TypeDefinition classDef = module.Types.First(d => d.Name == "CellRule" && d.Namespace == "RWG2.Rules");

        TypeReference dictoinaryType = module.Import(typeof(System.Collections.Generic.Dictionary<,>));

        TypeReference floatRef = module.Import(typeof(float));
        TypeReference stringRef = module.Import(typeof(string));


        GenericInstanceType giDictionaryType1 = new GenericInstanceType(dictoinaryType);
        giDictionaryType1.GenericArguments.Add(floatRef);
        giDictionaryType1.GenericArguments.Add(stringRef);

        GenericInstanceType giDictionaryType2 = new GenericInstanceType(dictoinaryType);
        giDictionaryType2.GenericArguments.Add(floatRef);
        giDictionaryType2.GenericArguments.Add(giDictionaryType1);


        AddFieldDefinition(classDef, "HubRulesGridDic", FieldAttributes.Public, giDictionaryType2);

        TypeReference tr = module.Import(typeof(float));
        TypeDefinition hubRule = module.Types.First(d => d.Name == "HubRule" && d.Namespace == "RWG2.Rules");
        AddFieldDefinition(hubRule, "HubOffsetX", FieldAttributes.Public, tr);
        AddFieldDefinition(hubRule, "HubOffsetY", FieldAttributes.Public, tr);

        tr = module.Import(typeof(bool));
        AddFieldDefinition(hubRule, "IsSetHubOffset", FieldAttributes.Public, tr);


    }

    public void AddFieldDefinition(TypeDefinition classDef, string fieldName, FieldAttributes fieldAttribute, TypeReference fieldType)
    {
        FieldDefinition fd = new FieldDefinition(fieldName, fieldAttribute, fieldType);
        classDef.Fields.Add(fd);

    }
    public void AddFieldDefinition(TypeDefinition classDef, string fieldName, FieldAttributes fieldAttribute, TypeReference fieldType, object constantvalue)
    {
        FieldDefinition fd = new FieldDefinition(fieldName, fieldAttribute, fieldType);
        fd.Constant = constantvalue;
        classDef.Fields.Add(fd);

    }
    private void SetFieldToPublic(FieldDefinition field)
    {
        field.IsFamily = false;
        field.IsPrivate = false;
        field.IsPublic = true;

    }
    private void SetFieldToPublic(TypeDefinition classType, string fieldName)
    {
        SetFieldToPublic(classType.Fields.First(d => d.Name == fieldName));
    }
}
