﻿using System;
using System.Linq;
using System.Xml;
using System.Collections.Generic;
using UnityEngine;
using System.Text;

public static class CellRuleHubPositionSetMethod
{
    public static bool IsSetHubOffset(RWG2.HubCell hubCell)
    {
        if (hubCell.HubRule == null)
        {
            return false;
        }
        return hubCell.HubRule.IsSetHubOffset;
    }
    public static Vector2 GetHubPositionFixRect(RWG2.HubCell hubCell,ref UnityEngine.Rect boundingBox,int width)
    {
        Vector2 ret = GetHubPosition(hubCell);
        if (boundingBox.x > ret.x - width)
        {
            float dec = boundingBox.x - (ret.x - width);
            boundingBox.x -= dec;
            boundingBox.width += dec;
        }
        if (boundingBox.y > ret.y - width)
        {
            float dec = boundingBox.y - (ret.y - width);
            boundingBox.y -= dec;
            boundingBox.height += dec;
        }
        if (boundingBox.x + boundingBox.width < ret.x + width)
        {
            float add = (ret.x + width) - (boundingBox.x + boundingBox.width);
            boundingBox.width += add;
        }
        if (boundingBox.y + boundingBox.height < ret.y + width)
        {
            float add = (ret.y + width) - (boundingBox.x + boundingBox.height);
            boundingBox.height += add;
        }

        return ret;

    }
    public static Vector2 GetHubPosition(RWG2.HubCell hubCell)
    {

        return new Vector2(hubCell.CenterInWorld.x + hubCell.HubRule.HubOffsetX, hubCell.CenterInWorld.y + hubCell.HubRule.HubOffsetY);
    }
    public static List<Vector2> GetHubPositionList(RWG2.HubCell hubCell)
    {
        List<Vector2> ret = new List<Vector2>();
        ret.Add(GetHubPosition(hubCell));
        return ret;
    }

    public static void AddHubRuleGridPos(RWG2.Rules.CellRule cellRule,string hubRuleName, XmlElement element)
    {
        if (element.HasAttribute("GridPosition"))
        {
            string[] stArray = element.GetAttribute("GridPosition").Split(new char[] { ',' }, StringSplitOptions.RemoveEmptyEntries);
            
            float x = float.Parse(stArray[0]);
            float y = float.Parse(stArray[1]);
            if(cellRule.HubRulesGridDic == null)
            {
                cellRule.HubRulesGridDic = new Dictionary<float, Dictionary<float, string>>();
            }
            if(!cellRule.HubRulesGridDic.ContainsKey(x))
            {
                cellRule.HubRulesGridDic.Add(x,new Dictionary<float, string>());
            }
            cellRule.HubRulesGridDic[x].Add(y, hubRuleName);
        }

    }
    public static bool GetGridHubRule(RWG2.HubCell hubCell, Vector2 grid)
    {
        RWG2.Rules.CellRule cellRule = hubCell.CellRule;
        if (cellRule != null)
        {
            if (cellRule.HubRulesGridDic != null)
            {
                if (cellRule.HubRulesGridDic.ContainsKey(grid.x) && cellRule.HubRulesGridDic[grid.x].ContainsKey(grid.y))
                {
                    string hubRuleName =cellRule.HubRulesGridDic[grid.x][grid.y];
                    if (RWG2.Rules.RWGRules.Instance.HubRules.ContainsKey(hubRuleName))
                    {
                        hubCell.HubRule = RWG2.Rules.RWGRules.Instance.HubRules[hubRuleName];
                        return true;
                    }
                }
            }
        }
        return false;

    }
    public static void SetHubRuleAttribute(RWG2.Rules.HubRule hubRule, XmlElement element)
    {
        if (element.Name.Equals("HubOffset") && element.HasAttribute("value"))
        {
            string[] stArray = element.GetAttribute("value").Split(new char[] { ',' }, StringSplitOptions.RemoveEmptyEntries);
            hubRule.IsSetHubOffset = true;
            hubRule.HubOffsetX = float.Parse(stArray[0]);
            hubRule.HubOffsetY = float.Parse(stArray[1]);
        }
    }
    public static int floodFillSet(RWG2.HubGenerators.HGSockets hgSocket, Vector2 seedPos, ref List<global::Vector2i> hubNodes)
    {
        RWG2.HubCell.HubPathNode hubPathNodeByWorldPos = hgSocket.hubCell.GetHubPathNodeByWorldPos((int)seedPos.x, (int)seedPos.y);
        hgSocket.hubCell.hubStartNodeIndex = hubPathNodeByWorldPos.Index;
        RWFill(hgSocket, hubPathNodeByWorldPos, ref hubNodes);
        if (hubNodes.Count < 300)
        {
            for (int i = 0; i < hubNodes.Count; i++)
            {
                RWG2.HubCell.HubPathNode hubPathNodeByWorldPos2 = hgSocket.hubCell.GetHubPathNodeByWorldPos(hubNodes[i].x, hubNodes[i].y);
                if (hubPathNodeByWorldPos2.SocketTypeMain == RWG2.HubCell.SocketType.OutOfBounds)
                {
                }
                else if (hubPathNodeByWorldPos2.SocketTypeMain == RWG2.HubCell.SocketType.NearCellEdge)
                {
                }
                else if (hubPathNodeByWorldPos2.SocketTypeMain == RWG2.HubCell.SocketType.Water)
                {
                }
                else if (hubPathNodeByWorldPos2.SocketTypeMain != RWG2.HubCell.SocketType.Road)
                {
                    if (hubPathNodeByWorldPos2.SocketTypeMain == RWG2.HubCell.SocketType.CountryRoad)
                    {
                    }
                    else
                    {
                        hubPathNodeByWorldPos2.SocketTypeMain = RWG2.HubCell.SocketType.None;
                        hgSocket.hubCell.SetHubPathNode(hubPathNodeByWorldPos2);
                    }
                }
            }
        }
        return hubNodes.Count;
    }
    public static void RWFillAdd(RWG2.HubGenerators.HGSockets hgSocket, RWG2.HubCell.HubPathNode A_1, ref List<global::Vector2i> A_2, int A_3)
    {
        if (!hgSocket.UZ.Contains(A_1.Index))
        {
            if (!A_2.Contains(A_1.Index))
            {
                hgSocket.UZ.Add(A_1.Index);
                float num = (float)(hgSocket.TZ * 2) / 21f * ((float)(hgSocket.TZ * 2) / 21f);
                if ((float)hgSocket.UZ.Count <= num)
                {
                    if ((float)A_2.Count <= num)
                    {
                        if (Vector2.Distance(hgSocket.hubCell.GetHubPathNodeByGridPos(hgSocket.hubCell.hubStartNodeIndex.x, hgSocket.hubCell.hubStartNodeIndex.y).Area.center, A_1.Area.center) <= (float)hgSocket.TZ)
                        {
                            if (A_1.Height >= 6)
                            {

                            }
                            else
                            {
                                A_1.Height = 6;
                            }
                            if (A_1.SocketTypeMain != RWG2.HubCell.SocketType.OutOfBounds)
                            {
                                if (A_1.SocketTypeMain == RWG2.HubCell.SocketType.Water)
                                {
                                    A_1.SocketTypeMain = RWG2.HubCell.SocketType.None;
                                    if (A_1.SocketTypeExtra == RWG2.HubCell.SocketType.Water)
                                    {
                                        A_1.SocketTypeExtra = RWG2.HubCell.SocketType.None;
                                    }
                                }
                                if (A_1.SocketTypeMain != RWG2.HubCell.SocketType.NearCellEdge)
                                {
                                    int def = (int)A_1.Height - A_3;
                                    int num2 = Mathf.Abs(def);
                                    int num3;
                                    if (hgSocket.YZ)
                                    {
                                        num3 = 3;
                                    }
                                    else
                                    {
                                        num3 = 6;
                                    }
                                    if (num2 <= num3)
                                    {

                                    }
                                    else
                                    {
                                        if (def < 0)
                                        {
                                            A_1.Height = (short)(A_3 - num3);
                                        }
                                    }
                                    if (A_1.SocketTypeMain != RWG2.HubCell.SocketType.None)
                                    {
                                        if (A_1.SocketTypeMain == RWG2.HubCell.SocketType.RoadsideLot)
                                        {
                                        }
                                        else
                                        {
                                            if (A_1.SocketTypeMain == RWG2.HubCell.SocketType.Hub)
                                            {
                                                A_2.Add(A_1.Index);
                                                return;
                                            }
                                            return;
                                        }
                                    }
                                    A_2.Add(A_1.Index);

                                    return;
                                }
                            }
                        }
                        return;
                    }
                }
                return;
            }
        }
    }

    public static void RWFill(RWG2.HubGenerators.HGSockets hgSocket, RWG2.HubCell.HubPathNode A_1, ref List<global::Vector2i> A_2)
    {
        if (A_1.Height >= 6)
        {

        }
        else
        {
            A_1.Height = 6;
        }
        RWFillAdd(hgSocket, A_1, ref A_2, A_1.Height);

        int startx = A_1.IndexX;
        int starty = A_1.IndexY;
        int widthHalf = (int)(hgSocket.TZ / 21f);
        for (int i = 1; i <= widthHalf; i++)
        {
            int minX = i * -1 + startx;
            int maxX = i * 1 + startx;
            int minY = i * -1 + starty;
            int maxY = i * 1 + starty;
            for (int j= minX; j <= maxX; j++)
            {
                if (j == minX)
                {
                    RWG2.HubCell.HubPathNode nodeSet = hgSocket.hubCell.GetHubPathNodeByGridPos(j, minY);
                    RWG2.HubCell.HubPathNode nodeHeight = hgSocket.hubCell.GetHubPathNodeByGridPos(j + 1, minY + 1);
                    RWFillAdd(hgSocket, nodeSet, ref A_2, nodeHeight.Height);
                    nodeSet = hgSocket.hubCell.GetHubPathNodeByGridPos(j, maxY);
                    nodeHeight = hgSocket.hubCell.GetHubPathNodeByGridPos(j + 1, maxY - 1);
                    RWFillAdd(hgSocket, nodeSet, ref A_2, nodeHeight.Height);

                }
                else if (j == maxX)
                {
                    RWG2.HubCell.HubPathNode nodeSet = hgSocket.hubCell.GetHubPathNodeByGridPos(j, minY);
                    RWG2.HubCell.HubPathNode nodeHeight = hgSocket.hubCell.GetHubPathNodeByGridPos(j - 1, minY + 1);
                    RWFillAdd(hgSocket, nodeSet, ref A_2, nodeHeight.Height);
                    nodeSet = hgSocket.hubCell.GetHubPathNodeByGridPos(j, maxY);
                    nodeHeight = hgSocket.hubCell.GetHubPathNodeByGridPos(j - 1, maxY - 1);
                    RWFillAdd(hgSocket, nodeSet, ref A_2, nodeHeight.Height);

                }
                else
                {
                    RWG2.HubCell.HubPathNode nodeSet = hgSocket.hubCell.GetHubPathNodeByGridPos(j, minY);
                    RWG2.HubCell.HubPathNode nodeHeight = hgSocket.hubCell.GetHubPathNodeByGridPos(j, minY + 1);
                    RWFillAdd(hgSocket, nodeSet, ref A_2, nodeHeight.Height);
                    nodeSet = hgSocket.hubCell.GetHubPathNodeByGridPos(j, maxY);
                    nodeHeight = hgSocket.hubCell.GetHubPathNodeByGridPos(j, maxY - 1);
                    RWFillAdd(hgSocket, nodeSet, ref A_2, nodeHeight.Height);
                }
            }
            for (int j = minY; j <= maxY; j++)
            {
                if (j == minY)
                {
                    RWG2.HubCell.HubPathNode nodeSet = hgSocket.hubCell.GetHubPathNodeByGridPos(minX, j);
                    RWG2.HubCell.HubPathNode nodeHeight = hgSocket.hubCell.GetHubPathNodeByGridPos(minX + 1, j + 1);
                    RWFillAdd(hgSocket, nodeSet, ref A_2, nodeHeight.Height);
                    nodeSet = hgSocket.hubCell.GetHubPathNodeByGridPos(maxX, j);
                    nodeHeight = hgSocket.hubCell.GetHubPathNodeByGridPos(maxX - 1, j + 1);
                    RWFillAdd(hgSocket, nodeSet, ref A_2, nodeHeight.Height);

                }
                else if (j == maxY)
                {
                    RWG2.HubCell.HubPathNode nodeSet = hgSocket.hubCell.GetHubPathNodeByGridPos(minX, j);
                    RWG2.HubCell.HubPathNode nodeHeight = hgSocket.hubCell.GetHubPathNodeByGridPos(minX + 1, j - 1);
                    RWFillAdd(hgSocket, nodeSet, ref A_2, nodeHeight.Height);
                    nodeSet = hgSocket.hubCell.GetHubPathNodeByGridPos(maxX, j);
                    nodeHeight = hgSocket.hubCell.GetHubPathNodeByGridPos(maxX - 1, j - 1);
                    RWFillAdd(hgSocket, nodeSet, ref A_2, nodeHeight.Height);

                }
                else
                {
                    RWG2.HubCell.HubPathNode nodeSet = hgSocket.hubCell.GetHubPathNodeByGridPos(minX,j);
                    RWG2.HubCell.HubPathNode nodeHeight = hgSocket.hubCell.GetHubPathNodeByGridPos(minX + 1,j);
                    RWFillAdd(hgSocket, nodeSet, ref A_2, nodeHeight.Height);
                    nodeSet = hgSocket.hubCell.GetHubPathNodeByGridPos(maxX,j);
                    nodeHeight = hgSocket.hubCell.GetHubPathNodeByGridPos(maxX - 1,j);
                    RWFillAdd(hgSocket, nodeSet, ref A_2, nodeHeight.Height);
                }
            }
        }
    }

}
